//
use crate::s;

/// The URL of a remote repository
#[derive(Clone, Debug, derive_more::Display, PartialEq, Eq, PartialOrd, Ord)]
#[display("https://{user}:{password}@{host}/{fullname}.git")]
pub struct RemoteUrl {
    pub host: String,
    pub fullname: String,
    pub path: String,
    pub user: String,
    pub password: String,
}

impl RemoteUrl {
    pub fn parse(url: impl Into<String>) -> Option<Self> {
        let parsed = ::git_url_parse::GitUrl::parse(&url.into()).ok()?;
        Some(Self {
            host: parsed.host.unwrap_or_default(),
            fullname: parsed.fullname,
            path: parsed.path,
            user: parsed.user.unwrap_or_default(),
            password: parsed.token.unwrap_or_default(),
        })
    }
}
impl TryFrom<gix::Url> for RemoteUrl {
    type Error = ();

    fn try_from(url: gix::Url) -> Result<Self, Self::Error> {
        let password = url.password().map(ToOwned::to_owned).unwrap_or_default();
        let parsed = ::git_url_parse::GitUrl::parse(&s!(url)).map_err(|_| ())?;
        Ok(Self {
            host: parsed.host.unwrap_or_default(),
            fullname: parsed.fullname,
            path: parsed.path,
            user: parsed.user.unwrap_or_default(),
            password,
        })
    }
}
impl RemoteUrl {
    pub fn matches(&self, other: &Self) -> bool {
        tracing::debug!(?self, ?other);
        self.host.eq(&other.host)
            && self.fullname.eq(&other.fullname)
            && self.password.eq(&other.password)
    }
}
