//
use crate::newtype;

newtype!(
    WebhookAuth,
    String,
    derive_more::Display,
    r"The unique token authorisation for the webhook.

Each monitored repository has it's own unique token, and it is different each time `git-next` runs."
);
impl WebhookAuth {
    /// Parses the authorisation string as a `Ulid` to create a `WebhookAuth`.
    ///
    /// # Errors
    ///
    /// Will return an `Err` if the authorisation string is not a valid `Ulid`.
    pub fn try_new(authorisation: &str) -> Result<Self, ulid::DecodeError> {
        use std::str::FromStr as _;
        let id = ulid::Ulid::from_str(authorisation)?.to_string();
        tracing::info!("Parse auth token: {}", id);
        Ok(Self(id))
    }

    #[must_use]
    pub fn generate() -> Self {
        Self(ulid::Ulid::new().to_string())
    }

    #[must_use]
    pub fn header_value(&self) -> String {
        format!("Basic {self}")
    }
}
