//
use crate::core::config::{BranchName, RepoBranches};
use derive_more::Constructor;

#[derive(
    Clone,
    Debug,
    Constructor,
    PartialEq,
    Eq,
    derive_with::With,
    serde::Serialize,
    serde::Deserialize,
)]
pub struct Push {
    pub branch: BranchName,
    sha: String,
    message: String,
}
impl Push {
    pub fn branch(&self, repo_branches: &RepoBranches) -> Option<Branch> {
        if self.branch == repo_branches.main() {
            return Some(Branch::Main);
        }
        if self.branch == repo_branches.next() {
            return Some(Branch::Next);
        }
        if self.branch == repo_branches.dev() {
            return Some(Branch::Dev);
        }
        tracing::warn!(branch = %self.branch, "Unexpected branch");
        None
    }

    #[must_use]
    pub fn sha(&self) -> &str {
        &self.sha
    }

    #[must_use]
    pub fn message(&self) -> &str {
        &self.message
    }
}

#[derive(Clone, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
pub enum Branch {
    Main,
    Next,
    Dev,
}
