//
pub type Result<T> = core::result::Result<T, Error>;

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("network")]
    Network(#[from] kxio::net::Error),

    #[error("reqwest")]
    Reqwest(#[from] kxio::net::RequestError),

    // #[error("header")]
    // Header(#[from] http::header::InvalidHeaderValue),

    // #[error("parse url")]
    // UrlParse(#[from] url::ParseError),
    #[error("failed to register: {0}")]
    FailedToRegister(String),

    #[error("failed to unregister: {0}")]
    FailedToUnregister(String),

    #[error("network response was empty")]
    NetworkResponseEmpty,

    #[error("repo config not loaded")]
    NoRepoConfig,

    #[error("failed to notify self of loaded config")]
    FailedToNotifySelf(String),

    #[error("(de)serialisation")]
    Serde(#[from] serde_json::error::Error),

    #[error("unknown branch: {0}")]
    UnknownBranch(String),

    #[error("failed to list: {0}")]
    FailedToList(String),
}
