//
use crate::{
    core::BranchName,
    git::{self, repository::open::OpenRepositoryLike},
    Result,
};

#[derive(Clone, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
pub enum Force {
    No,
    From(git::GitRef),
}
impl std::fmt::Display for Force {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::No => write!(f, "fast-forward"),
            Self::From(from) => write!(f, "force-if-from:{from}"),
        }
    }
}

/// Resets the position of a branch in the remote repo
///
/// Performs a 'git fetch' first to ensure we have up-to-date branch positions before
/// performing `git push`.
///
/// # Errors
///
/// Will return an `Err` if their is no remote fetch defined in .git/config, or
/// if there are any network connectivity issues with the remote server.
pub fn reset(
    open_repository: &dyn OpenRepositoryLike,
    repo_details: &git::RepoDetails,
    branch_name: &BranchName,
    to_commit: &git::GitRef,
    force: &git::push::Force,
) -> Result<()> {
    open_repository.fetch()?;
    open_repository.push(repo_details, branch_name, to_commit, force)
}
