//
use crate::core::git::{ForgeLike, RepoDetails};

#[cfg(feature = "forgejo")]
use crate::forges::forgejo::ForgeJo;

#[cfg(feature = "github")]
use crate::forges::github::Github;

use kxio::net::Net;

#[derive(Clone, Debug)]
pub struct Forge;

impl Forge {
    pub fn create(repo_details: RepoDetails, net: Net) -> Box<dyn ForgeLike> {
        match repo_details.forge.forge_type() {
            #[cfg(feature = "forgejo")]
            crate::core::ForgeType::ForgeJo => Box::new(ForgeJo::new(repo_details, net)),
            #[cfg(feature = "github")]
            crate::core::ForgeType::GitHub => Box::new(Github::new(repo_details, net)),
            _ => {
                drop(repo_details);
                drop(net);
                unreachable!();
            }
        }
    }
}

#[cfg(test)]
pub mod tests;
