//
use crate::core::{git, server::RepoListenUrl, WebhookId};
use kxio::net::Net;

use crate::forges::forgejo::webhook::Hook;

pub async fn list(
    repo_details: &git::RepoDetails,
    repo_listen_url: &RepoListenUrl,
    net: &Net,
) -> git::forge::webhook::Result<Vec<WebhookId>> {
    let mut ids: Vec<WebhookId> = vec![];
    let hostname = &repo_details.forge.hostname();
    let repo_path = &repo_details.repo_path;
    let mut page = 1;
    loop {
        use secrecy::ExposeSecret;
        let token = &repo_details.forge.token().expose_secret();
        match net
            .get(format!(
                "https://{hostname}/api/v1/repos/{repo_path}/hooks?page={page}&token={token}"
            ))
            .send()
            .await
        {
            Ok(response) => {
                if let Ok(list) = response.json::<Vec<Hook>>().await {
                    if list.is_empty() {
                        return Ok(ids);
                    }
                    for hook in list {
                        if let Some(existing_url) = hook.config.get("url") {
                            if existing_url.starts_with(&repo_listen_url.to_string()) {
                                ids.push(hook.id());
                            }
                        }
                    }
                    page += 1;
                }
            }
            Err(e) => {
                return Err(git::forge::webhook::Error::Network(e));
            }
        };
    }
}
