//
use crate::core::{git, WebhookId};
use crate::forges::github;

// https://docs.github.com/en/rest/repos/webhooks?apiVersion=2022-11-28#delete-a-repository-webhook
pub async fn unregister(
    github: &github::Github,
    webhook_id: &WebhookId,
) -> git::forge::webhook::Result<()> {
    let net = &github.net;
    let repo_details = &github.repo_details;
    let hostname = repo_details.forge.hostname();
    net.delete(format!(
        "https://api.{hostname}/repos/{}/hooks/{}",
        repo_details.repo_path, webhook_id
    ))
    .headers(github::webhook::headers(repo_details.forge.token()))
    .send()
    .await
    .map_err(|e| git::forge::webhook::Error::FailedToRegister(e.to_string()))
    .map(|_| ())
}
