pub type MessageResult<T = (), E = IoSendError> = std::result::Result<T, E>;

#[derive(Debug, derive_more::with_trait::Display, derive_more::From)]
pub enum IoSendError {
    IO(#[from] std::io::Error),
    Send,
    Other(#[from] anyhow::Error),
    Signal(String),
}
impl std::error::Error for IoSendError {}

impl<T, E> From<kameo::error::SendError<T, E>> for IoSendError {
    fn from(_value: kameo::error::SendError<T, E>) -> Self {
        Self::Send
    }
}
impl<T> From<tokio::sync::mpsc::error::SendError<T>> for IoSendError {
    fn from(value: tokio::sync::mpsc::error::SendError<T>) -> Self {
        Self::Signal(value.to_string())
    }
}
