//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, warn};

use crate::{
    repo::{
        do_send,
        messages::{CheckCIStatus, ReceiveCIStatus},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<CheckCIStatus> for RepoActor {
    type Reply = Result<()>;

    async fn handle(
        &mut self,
        msg: CheckCIStatus,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        crate::repo::logger(self.log.as_ref(), "start: CheckCIStatus").await;
        let next = msg.peel();
        self.post_repo_update(RepoUpdate::CheckingCI).await?;
        // get the status - pass, fail, pending (all others map to fail, e.g. error)
        match self.forge.commit_status(&next).await {
            Ok(status) => {
                debug!("got status: {status:?}");
                do_send(
                    &ctx.actor_ref(),
                    ReceiveCIStatus::new((next, status)),
                    self.log.as_ref(),
                )
                .await?;
            }
            Err(err) => warn!(?err, "fetching commit status"),
        }
        Ok(())
    }
}
