//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, instrument, warn};

use crate::{
    core::git,
    repo::{
        do_send, logger,
        messages::{CloneRepo, LoadConfigFromRepo, RegisterWebhook},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<CloneRepo> for RepoActor {
    type Reply = Result<()>;
    #[instrument(name = "RepoActor::CloneRepo", skip_all, fields(repo = %self.repo_details))]
    async fn handle(
        &mut self,
        _msg: CloneRepo,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        logger(self.log.as_ref(), "Handler: CloneRepo: start").await;
        self.post_repo_update(RepoUpdate::Opening {
            repo_actor: ctx.actor_ref(),
        })
        .await?;
        debug!("Handler: CloneRepo: start");
        match git::repository::open(&*self.repository_factory, &self.repo_details) {
            Ok(repository) => {
                logger(self.log.as_ref(), "open okay").await;
                debug!("open okay");
                self.post_repo_update(RepoUpdate::Opened).await?;
                self.open_repository.replace(repository);
                if self.repo_details.repo_config.is_none() {
                    debug!("no repo config, need to load from repo");
                    do_send(&ctx.actor_ref(), LoadConfigFromRepo, self.log.as_ref()).await?;
                } else {
                    debug!("have repo config");
                    do_send(&ctx.actor_ref(), RegisterWebhook, self.log.as_ref()).await?;
                }
            }
            Err(err) => {
                logger(self.log.as_ref(), "open failed").await;
                warn!("Could not open repo: {err:?}");
                self.alert_tui(err.to_string()).await?;
            }
        }
        debug!("Handler: CloneRepo: finish");
        Ok(())
    }
}
