//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, instrument};

use crate::{
    core::git::UserNotification,
    repo::{
        do_send, load,
        messages::{LoadConfigFromRepo, ReceiveRepoConfig},
        notify_user, RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<LoadConfigFromRepo> for RepoActor {
    type Reply = Result<()>;

    #[instrument(name = "Repocrate::repo::LoadConfigFromRepo", skip_all, fields(repo = %self.repo_details))]
    async fn handle(
        &mut self,
        _msg: LoadConfigFromRepo,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        debug!("Handler: LoadConfigFromRepo: start");
        self.post_repo_update(RepoUpdate::LoadingConfigFromRepo)
            .await?;
        let Some(open_repository) = &self.open_repository else {
            return Ok(());
        };
        match load::config_from_repository(&self.repo_details, &**open_repository).await {
            Ok(repo_config) => {
                do_send(
                    &ctx.actor_ref(),
                    ReceiveRepoConfig::new(repo_config),
                    self.log.as_ref(),
                )
                .await?;
            }
            Err(err) => {
                notify_user(
                    self.notify_user_recipient.as_ref(),
                    UserNotification::RepoConfigLoadFailure {
                        forge_alias: self.repo_details.forge.forge_alias().clone(),
                        repo_alias: self.repo_details.repo_alias.clone(),
                        reason: err.to_string(),
                    },
                    self.log.as_ref(),
                )
                .await?;
            }
        }
        debug!("Handler: LoadConfigFromRepo: finish");
        Ok(())
    }
}
