//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::instrument;

use crate::{
    repo::{
        do_send,
        messages::{ReceiveRepoConfig, RegisterWebhook},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<ReceiveRepoConfig> for RepoActor {
    type Reply = Result<()>;
    #[instrument(name = "RepoActor::ReceiveRepoConfig", skip_all, fields(repo = %self.repo_details, branches = ?msg))]
    async fn handle(
        &mut self,
        msg: ReceiveRepoConfig,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        let repo_config = msg.peel();
        self.post_repo_update(RepoUpdate::ReceiveRepoConfig {
            repo_config: repo_config.clone(),
        })
        .await?;
        self.repo_details.repo_config.replace(repo_config);
        self.update_tui_branches().await?;
        do_send(&ctx.actor_ref(), RegisterWebhook, self.log.as_ref()).await?;
        Ok(())
    }
}
