//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, error};

use crate::{
    core::git::UserNotification,
    repo::{
        do_send,
        messages::{RegisterWebhook, WebhookRegistered},
        notify_user, RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<RegisterWebhook> for RepoActor {
    type Reply = Result<()>;

    async fn handle(
        &mut self,
        _msg: RegisterWebhook,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        if self.webhook_id.is_none() {
            self.post_repo_update(RepoUpdate::RegisteringWebhook)
                .await?;
            debug!("registering webhook");
            match self
                .forge
                .register_webhook(&self.listen_url.repo_url(
                    self.repo_details.forge.forge_alias().clone(),
                    self.repo_details.repo_alias.clone(),
                ))
                .await
            {
                Ok(registered_webhook) => {
                    debug!(?registered_webhook, "webhook registered");
                    do_send(
                        &ctx.actor_ref(),
                        WebhookRegistered::from(registered_webhook),
                        self.log.as_ref(),
                    )
                    .await?;
                }
                Err(err) => {
                    error!(?err, "failed to register webhook");
                    notify_user(
                        self.notify_user_recipient.clone().as_ref(),
                        UserNotification::WebhookRegistration {
                            forge_alias: self.repo_details.forge.forge_alias().clone(),
                            repo_alias: self.repo_details.repo_alias.clone(),
                            reason: err.to_string(),
                        },
                        self.log.as_ref(),
                    )
                    .await?;
                }
            }
        } else {
            self.alert_tui("already have a webhook id - cant register webhook")
                .await?;
        }
        Ok(())
    }
}
