//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, warn};

use crate::{
    repo::{messages::UnRegisterWebhook, RepoActor},
    server::actor::messages::RepoUpdate,
};

impl Message<UnRegisterWebhook> for RepoActor {
    type Reply = Result<()>;

    async fn handle(
        &mut self,
        _msg: UnRegisterWebhook,
        _ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        let Some(webhook_id) = self.webhook_id.take() else {
            return Ok(());
        };
        self.post_repo_update(RepoUpdate::UnregisteringWebhook)
            .await?;
        debug!("unregistering webhook");
        match self.forge.unregister_webhook(&webhook_id).await {
            Ok(()) => debug!("unregistered webhook"),
            Err(err) => warn!(?err, "unregistering webhook"),
        }
        debug!("unregistering webhook done");
        Ok(())
    }
}
