//
use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::instrument;

use crate::{
    repo::{
        do_send,
        messages::{ValidateRepo, WebhookRegistered},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Message<WebhookRegistered> for RepoActor {
    type Reply = Result<()>;

    #[instrument(name = "RepoActor::WebhookRegistered", skip_all, fields(repo = %self.repo_details, webhook_id = %msg.webhook_id()))]
    async fn handle(
        &mut self,
        msg: WebhookRegistered,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        self.post_repo_update(RepoUpdate::RegisteredWebhook).await?;
        self.webhook_id.replace(msg.webhook_id().clone());
        self.webhook_auth.replace(msg.webhook_auth().clone());
        do_send(
            &ctx.actor_ref(),
            ValidateRepo::new(self.message_token),
            self.log.as_ref(),
        )
        .await?;
        Ok(())
    }
}
