//
use color_eyre::Result;
use kameo::message::{Context, Message};

use crate::{
    server::actor::{
        messages::{ReceiveAppConfig, ReceiveValidAppConfig, ValidAppConfig},
        ServerActor,
    },
    tell,
};

impl Message<ReceiveAppConfig> for ServerActor {
    type Reply = Result<()>;

    #[allow(clippy::cognitive_complexity)]
    async fn handle(
        &mut self,
        msg: ReceiveAppConfig,
        ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        let Ok(socket_addr) = msg.listen_socket_addr() else {
            self.abort(ctx, "Unable to parse http.addr").await?;
            return Ok(());
        };
        let Some(server_storage) = self.server_storage(&msg) else {
            self.abort(ctx, "Server storage not available").await?;
            return Ok(());
        };
        if msg.listen().url().ends_with('/') {
            self.abort(ctx, "webhook.url must not end with a '/'")
                .await?;
            return Ok(());
        }
        tell!(
            "server",
            ctx.actor_ref(),
            ReceiveValidAppConfig::new(ValidAppConfig::new(
                msg.peel(),
                socket_addr,
                server_storage,
            ))
        )?;
        Ok(())
    }
}
