use color_eyre::Result;
use kameo::message::{Context, Message};
use tracing::{debug, info};

use crate::{
    repo::messages::UnRegisterWebhook,
    server::actor::{messages::Shutdown, ServerActor},
    tell,
};

impl Message<Shutdown> for ServerActor {
    type Reply = Result<()>;

    async fn handle(
        &mut self,
        _msg: Shutdown,
        _ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        for ((forge_alias, repo_alias), repo_actor_ref) in &self.repo_actors {
            debug!(%forge_alias, %repo_alias, "removing webhook");
            tell!(repo_actor_ref, UnRegisterWebhook)?;
            debug!(%forge_alias, %repo_alias, "removed webhook");
            repo_actor_ref.kill();
            info!(%forge_alias, %repo_alias, "killed repo actor");
        }
        debug!("server shutdown");
        if let Some(webhook_actor_ref) = self.webhook_actor_ref.take() {
            debug!("shutting down webhook");
            webhook_actor_ref.kill();
            debug!("webhook shutdown");
        }
        Ok(())
    }
}
