//
use std::collections::BTreeMap;

use crate::core::{ForgeAlias, RepoAlias};
use ratatui::{
    buffer::Buffer,
    layout::{Alignment, Direction, Layout, Rect},
    text::Line,
    widgets::{Block, Widget},
};

use crate::tui::{
    actor::{RepoState, UIRepoFilter},
    components::{repo::RepoWidget, HeightContraintLength},
};

pub struct ExpandedForgeWidget<'a> {
    pub forge_alias: &'a ForgeAlias,
    pub repos: &'a BTreeMap<RepoAlias, RepoState>,
    pub filter: &'a UIRepoFilter,
}
impl HeightContraintLength for ExpandedForgeWidget<'_> {
    fn height_constraint_length(&self) -> u16 {
        self.children()
            .iter()
            .map(HeightContraintLength::height_constraint_length)
            .sum::<u16>()
            + 2 // top title + bottom padding
    }
}
impl Widget for ExpandedForgeWidget<'_> {
    fn render(self, area: Rect, buf: &mut Buffer)
    where
        Self: Sized,
    {
        let block = Block::default().title_top(
            Line::from(format!(" forge: {} ", self.forge_alias)).alignment(Alignment::Left),
        );
        let children = self.children();
        let layout = Layout::default()
            .direction(Direction::Vertical)
            .constraints(
                children
                    .iter()
                    .map(HeightContraintLength::height_constraint_length),
            )
            .split(block.inner(area));
        block.render(area, buf);
        children
            .into_iter()
            .enumerate()
            .for_each(|(i, w)| w.render(layout[i], buf));
    }
}
impl<'a> ExpandedForgeWidget<'a> {
    fn children(&self) -> Vec<RepoWidget<'a>> {
        self.repos
            .values()
            .filter(|repo_state| {
                if self.filter.is_empty() {
                    true
                } else {
                    let repo_alias = repo_state.repo_alias();
                    // eprintln!("--> {} ? {}", self.filter.as_str(), repo_alias);
                    repo_alias.contains(self.filter.as_str())
                }
            })
            .map(|repo_state| RepoWidget { repo_state })
            .collect::<Vec<_>>()
    }
}
