//
mod collapsed;
mod expanded;

use std::collections::BTreeMap;

use crate::core::{ForgeAlias, RepoAlias};
use collapsed::CollapsedForgeWidget;
use expanded::ExpandedForgeWidget;
use ratatui::{buffer::Buffer, layout::Rect, widgets::Widget};

use crate::tui::actor::{RepoState, UIRepoFilter, ViewState};

use super::HeightContraintLength;

pub struct ForgeWidget<'a> {
    pub forge_alias: &'a ForgeAlias,
    pub repos: &'a BTreeMap<RepoAlias, RepoState>,
    pub view_state: ViewState,
    pub filter: &'a UIRepoFilter,
}
impl HeightContraintLength for ForgeWidget<'_> {
    fn height_constraint_length(&self) -> u16 {
        match self.view_state {
            ViewState::Collapsed => CollapsedForgeWidget {
                forge_alias: self.forge_alias,
            }
            .height_constraint_length(),
            ViewState::Expanded => ExpandedForgeWidget {
                forge_alias: self.forge_alias,
                repos: self.repos,
                filter: self.filter,
            }
            .height_constraint_length(),
        }
    }
}
impl Widget for ForgeWidget<'_> {
    fn render(self, area: Rect, buf: &mut Buffer)
    where
        Self: Sized,
    {
        match self.view_state {
            ViewState::Collapsed => CollapsedForgeWidget {
                forge_alias: self.forge_alias,
            }
            .render(area, buf),
            ViewState::Expanded => ExpandedForgeWidget {
                forge_alias: self.forge_alias,
                repos: self.repos,
                filter: self.filter,
            }
            .render(area, buf),
        }
    }
}
