use std::string::ToString;

//
use crate::core::{RepoAlias, RepoBranches};
use ratatui::{
    layout::Alignment,
    style::{Color, Style, Stylize as _},
    text::{Line, Span},
    widgets::block::Title,
};

use crate::tui::actor::RepoMessage;

pub struct Identity<'a> {
    pub repo_alias: &'a RepoAlias,
    pub alert: Option<&'a str>,
    pub message: &'a RepoMessage,
    pub repo_branches: Option<&'a RepoBranches>,
}
impl<'a> Identity<'a> {
    pub const fn new(
        repo_alias: &'a RepoAlias,
        alert: Option<&'a str>,
        message: &'a RepoMessage,
        repo_branches: Option<&'a RepoBranches>,
    ) -> Self {
        Self {
            repo_alias,
            alert,
            message,
            repo_branches,
        }
    }
}
impl<'a> Identity<'a> {
    fn spans(self) -> Vec<Span<'a>> {
        let alert = self
            .alert
            .map(ToString::to_string)
            .map(|alert| alert.fg(Color::White).bg(Color::Red));
        let message = self.message;
        let main = self
            .repo_branches
            .map(RepoBranches::main)
            .map_or_else(|| "_".to_string(), |b| b.to_string());
        let next = self
            .repo_branches
            .map(RepoBranches::next)
            .map_or_else(|| "_".to_string(), |b| b.to_string());
        let dev = self
            .repo_branches
            .map(RepoBranches::dev)
            .map_or_else(|| "_".to_string(), |b| b.to_string());
        let mut spans = vec![" ".into()];
        match alert {
            None => spans.push(
                Span::from(self.repo_alias.to_string()).style(Style::default().fg(Color::Cyan)),
            ),
            Some(alert) => {
                spans.push(
                    Span::from(self.repo_alias.to_string())
                        .style(Style::default().fg(Color::White).bg(Color::Red)),
                );
                spans.push(" ".into());
                spans.push(alert);
            }
        }
        spans.push(" ".into());
        spans.push(format!("({main} -> {next} -> {dev}) ").into());
        spans.push(message.into());
        spans.push(" ".into());
        spans
    }
}
impl<'a> From<Identity<'a>> for Title<'a> {
    fn from(identity: Identity<'a>) -> Self {
        Self {
            content: Line {
                spans: identity.spans(),
                style: Style::reset(),
                alignment: None,
            },
            alignment: Some(Alignment::Left),
            position: None,
        }
    }
}
