//
use std::net::SocketAddr;

use color_eyre::Result;
use kameo::{
    actor::ActorRef,
    mailbox::unbounded::UnboundedMailbox,
    message::{Context, Message},
    Actor,
};
use router::WebhookRouterActor;

use crate::{
    default_on_actor_link_died, default_on_actor_panic, default_on_actor_start,
    default_on_actor_stop, IoSendError,
};

pub mod router;
mod server;

#[allow(clippy::module_name_repetitions)]
#[derive(Debug)]
pub struct WebhookActor {
    socket_addr: SocketAddr,
    message_receiver: ActorRef<WebhookRouterActor>,
}
impl WebhookActor {
    pub const fn new(
        socket_addr: SocketAddr,
        message_receiver: ActorRef<WebhookRouterActor>,
    ) -> Self {
        Self {
            socket_addr,
            message_receiver,
        }
    }
}

impl Actor for WebhookActor {
    type Mailbox = UnboundedMailbox<Self>;
    type Error = IoSendError;

    default_on_actor_start!(this, actor_ref);
    default_on_actor_panic!(this, actor_ref, err);
    default_on_actor_link_died!(this, actor_ref, id, reason);
    default_on_actor_stop!(this, actor_ref, reason);
}

#[derive(Debug)]
pub struct Start;
impl Message<Start> for WebhookActor {
    type Reply = Result<()>;

    async fn handle(&mut self, _msg: Start, _ctx: &mut Context<Self, Self::Reply>) -> Self::Reply {
        server::start(self.socket_addr, self.message_receiver.clone()).await?;
        Ok(())
    }
}
