use std::collections::HashMap;

use crate::server::actor::messages::ValidAppConfig;

use super::{repo_key::RepoKey, repo_state::RepoState};

#[derive(Debug, Default)]
pub struct DataState {
    repo: HashMap<RepoKey, RepoState>,
}
impl DataState {
    pub fn set_app_config(&mut self, app_config: &ValidAppConfig) {
        self.repo.clear();
        app_config.app_config.forge.iter().for_each(|(fa, fc)| {
            fc.repos.iter().for_each(|(ra, src)| {
                if let Some(rc) = src.repo_config() {
                    self.set_repo_state(
                        RepoKey::builder()
                            .forge_alias(fa.clone())
                            .repo_alias(ra.clone())
                            .build(),
                        RepoState::default().with_repo_branches(rc.branches().clone()),
                    );
                }
            });
        });
    }

    pub fn get_repo_state(&self, repo_key: &RepoKey) -> Option<&RepoState> {
        self.repo.get(repo_key)
    }

    pub fn set_repo_state(&mut self, repo_key: RepoKey, repo_state: RepoState) {
        self.repo.insert(repo_key, repo_state);
    }

    pub fn repos(&self) -> Vec<(&RepoKey, &RepoState)> {
        let mut repos = self.repo.iter().collect::<Vec<_>>();
        repos.sort_by(|(rka, _), (rkb, _)| rka.cmp(rkb));
        repos
    }
}
