use crossterm::event::{KeyCode, KeyEvent};
use kameo::actor::ActorRef;

use crate::{
    tell,
    xui::{
        state::{UiFocus, UiState},
        user_input::UserInterventionR,
        Xui,
    },
};

pub async fn handle_repo_list_key_event(
    key_event: KeyEvent,
    ui_state: &mut UiState,
    actor_ref: &ActorRef<Xui>,
) -> color_eyre::Result<()> {
    match key_event.code {
        // QUIT
        KeyCode::Char('q') => actor_ref.kill(),

        // SELECT REPO
        KeyCode::Char('j') | KeyCode::Down => {
            ui_state.table_state_mut().select_next();
        }
        KeyCode::Char('k') | KeyCode::Up => {
            ui_state.table_state_mut().select_previous();
        }

        // SCROLL VIEW
        KeyCode::Char('f') | KeyCode::PageDown => {
            ui_state.scroll_view_state_mut().scroll_page_down();
        }
        KeyCode::Char('b') | KeyCode::PageUp => {
            ui_state.scroll_view_state_mut().scroll_page_up();
        }
        KeyCode::Char('g') | KeyCode::Home => {
            ui_state.scroll_view_state_mut().scroll_to_top();
        }
        KeyCode::Char('G') | KeyCode::End => {
            ui_state.scroll_view_state_mut().scroll_to_bottom();
        }

        // USER INTERVENTION
        KeyCode::Char('r') => {
            tell!(actor_ref, UserInterventionR)?;
        }

        // EDIT FILTER
        KeyCode::Char('/') => {
            ui_state.set_focus(UiFocus::FilterInput);
        }

        _ => {}
    };

    Ok(())
}
