use kameo::message::{Context, Message};

use crate::{
    core::git::UserNotification,
    repo::messages::ValidateRepo,
    tell,
    xui::{
        state::{RepoStateStatus, UiFocus},
        Xui,
    },
};

use super::UserInterventionR;

impl Message<UserInterventionR> for Xui {
    type Reply = color_eyre::Result<()>;

    async fn handle(
        &mut self,
        _msg: UserInterventionR,
        _ctx: &mut Context<Self, Self::Reply>,
    ) -> Self::Reply {
        if let Some(repo_state) = self.state.selected_repo_state() {
            if let RepoStateStatus::UserIntervention { notification } = &repo_state.status() {
                match notification {
                    UserNotification::CICheckFailed { .. }
                    | UserNotification::RepoConfigLoadFailure { .. }
                    | UserNotification::WebhookRegistration { .. } => {
                        if let Some(repo_actor) = repo_state.repo_actor() {
                            tell!(repo_actor, ValidateRepo::new(repo_state.message_token()))?;
                        }
                    }
                    UserNotification::DevNotBasedOnMain { .. } => {
                        self.state.ui.set_focus(UiFocus::DevRebaseConfirm);
                        self.draw()?;
                    }
                }
            }
        }

        Ok(())
    }
}
