use bon::Builder;
use ratatui::{
    layout::Constraint,
    prelude::{Buffer, Rect},
    style::Style,
    text::Text,
    widgets::{Cell, Row, StatefulWidget, Table, TableState},
};

const MAX_STATUS_LENGTH: u16 = 20;

#[derive(Builder, Clone, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
pub struct RepoTableItem {
    forge_repo: String,
    status: String,
    style: Style,
}
impl RepoTableItem {
    const fn ref_array(&self) -> [&String; 2] {
        [&self.forge_repo, &self.status]
    }

    pub fn forge_repo(&self) -> &str {
        &self.forge_repo
    }
}

#[derive(Builder, Clone, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
pub struct RepoTable {
    items: Vec<RepoTableItem>,
}

impl StatefulWidget for RepoTable {
    type State = TableState;

    fn render(self, area: Rect, buf: &mut Buffer, state: &mut Self::State) {
        let header = ["Forge/Repo", "Status"]
            .into_iter()
            .map(Cell::from)
            .collect::<Row>()
            .height(2);
        let rows = self.items.iter().map(|data| {
            data.ref_array()
                .into_iter()
                .map(|content| Cell::from(Text::from(content.as_str())))
                .collect::<Row>()
                .height(1)
                .style(data.style)
        });
        #[allow(clippy::cast_possible_truncation)]
        let max_forge_repo_length: u16 = self
            .items
            .iter()
            .map(|data| data.forge_repo.len().clamp(11, 30) as u16)
            .max()
            .unwrap_or(11);

        let table = Table::new(
            rows,
            [
                Constraint::Length(max_forge_repo_length),
                Constraint::Length(MAX_STATUS_LENGTH),
            ],
        )
        .header(header)
        .highlight_spacing(ratatui::widgets::HighlightSpacing::Always)
        .highlight_symbol("> ");
        table.render(area, buf, state);
    }
}
